<?php

namespace Shopbydedi\Payment;

// TODO : find a way to remove references to those classes : Panier / Tpe
use Panier;
use Tpe;

use Symfony\Component\HttpFoundation\RedirectResponse;
use Shopbydedi\Payment\Exception\MissingConfig;
use PayPal\Api\Amount;
use PayPal\Api\Payer;
use PayPal\Api\Payment;
use PayPal\Api\RedirectUrls;
use PayPal\Api\Transaction;
use PayPal\Api\PaymentExecution;
use PayPal\Rest\ApiContext;
use PayPal\Auth\OAuthTokenCredential;

// use PayPal\Api\Details;
// use PayPal\Api\Item;
// use PayPal\Api\ItemList;

/**
 * Class Paypal
 * @package Shopbydedi\Payment
 */
class Paypal
{
    /**
     * @var PaypalConfig
     */
    protected $config;

    /**
     * Paypal constructor.
     *
     * @param $config
     *
     * @throws MissingConfig
     */
    public function __construct(PaypalConfig $config)
    {
        $this->config = $config;

        if ($this->config->isValid() === false) {
            throw new MissingConfig('Config provided for Paypal is not valid');
        }
    }


    /**
     * @param Panier $panier
     * @param $amount
     *
     * @throws \InvalidArgumentException
     */
    public function request($panier, $amount)
    {
        // $caddie = $panier->getProducts();

        $items = [];

        /*foreach ($caddie as $cad) {
            $item = (new Item())
                ->setName($cad['produit_nom_nom'])
                // TODO : change
                ->setCurrency('EUR')
                ->setQuantity($cad['quantite_order'])
                // TODO : change
                ->setPrice($cad['produit_prix_ht'] - $cad['produit_reduction_ht']);
            $items[] = $item;
        }*/

        // $itemList = (new ItemList())->setItems($items);

        // On vérifie le montant du subtotal pour qu'il corresponde
        /*$totalProduitHTPP = 0;

        foreach ($items as $i) {
            $totalProduitHTPP += $i->getPrice() * $i->getQuantity();
        }*/

        // $productsHT = $totalProduitHTPP;

        // $taxTotal = $isTva ? $productsTTC - $productsHT: 0;
        // $shippingTotal = $amount - ($productsHT + $taxTotal);


        /*$details = new Details();
        $details->setShipping($shippingTotal)
                ->setTax($taxTotal)
                ->setSubtotal($productsHT);
        */

        $amt = new Amount();
        $amt->setCurrency('EUR')        // TODO : change, make currency dynamic
            ->setTotal($amount)
            // ->setDetails($details)
        ;

        $reference = $panier->getReference();

        // $notifyUrl = $this->config->getUrlNotify();

        $transaction = new Transaction();
        $transaction->setAmount($amt)
                    // ->setItemList($itemList)
                    ->setDescription(sprintf('Transaction %s', $reference))
                    ->setInvoiceNumber($reference)
                    // ->setNotifyUrl($notifyUrl)
        ;

        return $this->processTransaction($transaction);
    }

    /**
     * @param Transaction $transaction
     *
     * @throws \InvalidArgumentException
     *
     * @return string
     */
    protected function processTransaction(Transaction $transaction)
    {
        $payer = new Payer();
        $payer->setPaymentMethod('paypal');

        $redirectUrls = new RedirectUrls();
        $redirectUrls
            ->setReturnUrl($this->config->getUrlNotify())
            ->setCancelUrl($this->config->getUrlFailure());


        $payment = new Payment();
        $payment->setIntent('sale')
                ->setPayer($payer)
                ->setRedirectUrls($redirectUrls)
                ->setTransactions(array($transaction));

        $approvalUrl = null;
        try {
            $payment->create($this->getApiContext());
        } catch (\Exception $e) {
            // Don't spit out errors or use "exit" like this in production code
            // dump($e);
        }

        $approvalUrl = $payment->getApprovalLink();

        if (empty($approvalUrl)) {
            // TODO : a revoir, pas top cet concaténation !
            $approvalUrl = $this->config->getUrlFailure() . '?error=paiementKO';
        }

        return $this->getFormTemplate($approvalUrl);
    }

    /**
     * @param $request
     *
     * @throws \InvalidArgumentException
     *
     * @return RedirectResponse
     */
    public function response($request)
    {
        $payment = $this->executePayment([
            'paymentId' => $request['paymentId'],
            'PayerID' => $request['PayerID'],
        ]);

        $transactions = $payment->getTransactions();
        $reference = $transactions[0]->getInvoiceNumber();

        $tpe = new Tpe;

        if ($payment->getState() === 'approved') {

            $orderId = $tpe->paiementOK($reference, $request);

            if (!empty($orderId)) {
                return new RedirectResponse($this->config->getUrlSuccess());
            }
        } else {
            $tpe->paiementKO($reference, $request);
        }

        return new RedirectResponse($this->config->getUrlFailure() . '?error=paiementKO');
    }

    /**
     * @param $request
     *
     * @throws \InvalidArgumentException
     *
     * @return Payment
     */
    public function executePayment($request)
    {
        $apiContext = $this->getApiContext();
        // Get the payment Object by passing paymentId
        // payment id was previously stored in session in
        // CreatePaymentUsingPayPal.php
        $paymentId = $request['paymentId'];
        $payment = Payment::get($paymentId, $apiContext);

        // ### Payment Execute
        // PaymentExecution object includes information necessary
        // to execute a PayPal account payment.
        // The payer_id is added to the request query parameters
        // when the user is redirected from paypal back to your site
        $execution = new PaymentExecution();
        $execution->setPayerId($request['PayerID']);

        try {
            // Execute the payment
            $result = $payment->execute($execution, $apiContext);

            // ResultPrinter::printResult("Executed Payment", "Payment", $payment->getId(), $execution, $result);

            try {
                $payment = Payment::get($paymentId, $apiContext);
            } catch (\Exception $ex) {
                // ResultPrinter::printError("Get Payment", "Payment", null, null, $ex);
            }
        } catch (\Exception $ex) {
            // ResultPrinter::printError("Executed Payment", "Payment", null, null, $ex);
        }

        // ResultPrinter::printResult("Get Payment", "Payment", $payment->getId(), null, $payment);

        return $payment;
    }


    /**
     * @return ApiContext
     */
    protected function getApiContext()
    {
        // ### Api context
        // Use an ApiContext object to authenticate
        // API calls. The clientId and clientSecret for the
        // OAuthTokenCredential class can be retrieved from
        // developer.paypal.com

        $apiContext = new ApiContext(
            new OAuthTokenCredential(
                $this->config->getClientId(),
                $this->config->getClientSecret()
            )
        );

        // Comment this line out and uncomment the PP_CONFIG_PATH
        // 'define' block if you want to use static file
        // based configuration

        $storageDir = $this->config->getStorageDir() . 'paypal/';

        if (is_dir($storageDir) === false) {
            mkdir($storageDir, 0775, true);
        }

        $apiContext->setConfig(
            array(
                'mode' => $this->config->getMode(),
                'log.LogEnabled' => true,
                'log.FileName' => $storageDir . 'PayPal.log',
                'log.LogLevel' => 'DEBUG', // PLEASE USE `FINE` LEVEL FOR LOGGING IN LIVE ENVIRONMENTS
                'validation.level' => 'log',
                'cache.enabled' => true,
                'cache.FileName' => $storageDir . 'auth.cache',
                // 'http.CURLOPT_CONNECTTIMEOUT' => 30
                // 'http.headers.PayPal-Partner-Attribution-Id' => '123123123'
            )
        );

        return $apiContext;
    }

    /**
     * @return string
     */
    public function getFormTemplate($url)
    {
        $template = '<a id="payer-paypal" href="'.$url.'">Paypal</a>';
        $template.= '<script>
                    var href = document.getElementById("payer-paypal").getAttribute("href");
                    window.location.href = href;
                    </script>';
        return $template;
    }
}
